﻿# CODE BLOCK BEGIN Global_Definitions 
from TSMaster import *
# import

import Databases as dbc

# 코드와 코드 설명이 있는 DTC 코드표를 xlsx에 미리 저장해 두었다. 
# 제어기에서 받은 DTC를 코드표에서 찾아 설명을 화면에 출력한다.
# 표를 편리하게 다루기 위해 pandas를 임포트한다.
import pandas as pd

# 파일 경로를 다루기 편리하도록 pathlib을 임포트한다. 
from pathlib import Path  
 


# constant
# 코드를 읽기 쉽게 service id 등을 상수로 정의해 둔다.
# 실제로는 값을 변경할 수 있는 변수이다. 
# 변수 이름 앞에 k_ 를 붙여 상수로 취급한다는 표시를 한다.

## m_id(message id)
# 진단 통신 m_id 범위
# 0x700 부터 0x7FF 까지
k_m_id_diag_lowest = 0x700
k_m_id_diag_highest = 0x7FF

# 진단 통신에 어떤 m_id가 사용되는지를 확인하기 위해 
# m_id를 0x700 부터 0x7FF까지 1씩 증가해 가면서 진단 요청을 하는 "스캐닝"을 한다.
# 1회 스캐닝에 시간이 제법 걸린다. 스크립트 개발중 스캔닝을 반복한다. 
# 스캐닝 시간을 단축하기 위해 스캐닝 범위를 축소한다.  
# 축소된 범위를 0x7D0 부터 0x7DF 까지로 한다.
k_m_id_scan_start = 0x7D0 - 1   # 먼저 m_id를 1 증가하고 계산을 하기 때문에 
k_m_id_scan_stop  = 0x7DF       

# 진단 통신 m_id의 전체  범위를 스캐닝할 때 아래 두 줄을 언케멘트한다. 
# k_m_id_scan_start = k_m_id_diag_lowest - 1    # 0x700
# k_m_id_scan_stop  = k_m_id_diag_highest       # 0x7FF

# funcational request의 message id
k_m_id_func = 0x7DF

# esc의 physical request message id
k_m_id_esc_diag_req = 0x7D1

# esc의 physical response message id
k_m_id_esc__diag_resp = 0x7D9


## s_id(serivce id)
# 진단 통신의 긍정 응답은 진단 서비스(s_id: serive id)에 
# 0x40을 더한 값으로 시작한다.
k_offset_positive_resp = 0x40

# 아래 서비스들로 스캐닝을 한다.
#    readDTC, readDataById

# read DTC의 service id = 0x19
k_req_s_id_readDtc = 0x19
k_resp_s_id_readDtc = k_req_s_id_readDtc + k_offset_positive_resp

# read ECU Id service id = 0x22
k_req_s_id_readDataById = 0x22
k_resp_s_id_readDataById = k_req_s_id_readDataById + k_offset_positive_resp


## 코드 적용 대상 설정 
# can or can_fd
# can과 can_fd는 메시지 전송할 때 설정이 조금 다르다.
# 아래 스위치를 변경하여,이 코드를 can에도 can_fd에도 사용할 수 있도록 한다.
k_is_can_fd = True

# 이 코드를 can에 사용할 때는 아래 줄을 언케멘트한다. 
# k_is_can_fd = False

# UDS의 s_id와 현대자동차가 사용했던 s_id가 다르다.
# 아래 스위치를 변경하여, 이 코드를 UDS에도 예전 현대자동차 s_id도
# 사용할 수 있도록 한다.  
# uds or old hmc
k_is_uds = True
# k_is_uds = False


## request message
# UDS 메시지와 old hmc로 나눈다.

## old hmc
k_req_extDiag = [0x02, 0x10, 0x90, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA]
k_req_defDiag = [0x02, 0x10, 0x81, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA]

k_req_readEcuId = [0x02, 0x1A, 0x80] # reportECUIdentificationDataTable = 80h 

k_req_readSwVer = [0x02, 0x1A, 0x8D] # report ECU Software Number = 8Dh 

k_req_readDtc = [0x04, 0x19, 0x00, 0xFF, 0x00, 0xAA, 0xAA, 0xAA]

k_req_testerPresent = [0x02, 0x3E, 0x00, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA]

# transport protocol flow control
# 0x30 0x00 0x00
#     0x30에서 3: flow control structure
#     0x30에서 0: flow status. 0 = continue to send
#     0x00: Block Size. 0 = no limit for block size
#     0x00: Stmin. 0 = 0msec wait time
k_flow_control = [0x30, 0x00, 0x00, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA]


## UDS
if k_is_uds:
    # Extended Diag Session
    # diagSessionControl: 0x10
    # sessionType: 0x03 = ExtendedDiagSession
    # 진단 명령어: [0x10][0x03]
    # CAN 데이터: [0x02][0x10][0x03][0xAA][0xAA][0xAA][0xAA][0xAA]   
    #    진단 명령어 길이를 CAN 데이터 앞에 포함한다.
    #    메시지의 빈 공간을 0xAA (패딩 바이트)로 채운다. 
    k_req_extDiag = [0x02, 0x10, 0x03, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA] 
    k_req_defDiag = [0x02, 0x10, 0x01, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA]     
    
    # Read ECU ID
    # readDataByLocalID: 0x22
    # DID
    #     0xF100 = ECU ID
    #     0x1000 = Software Version
    # 진단 명령: [0x22][0xF1][0x00]
    # CAN 데이터: [0x03][0x22][0xF1][0x00][0xAA][0xAA][0xAA][0xAA] 
    k_req_readEcuId = [0x03, 0x22, 0xF1, 0x00, 0xAA, 0xAA, 0xAA, 0xAA] 
    k_req_readSwVer = [0x03, 0x22, 0x10, 0x00, 0xAA, 0xAA, 0xAA, 0xAA] 
    k_req_readDataById = [0x03, 0x22, 0x00, 0x00, 0xAA, 0xAA, 0xAA, 0xAA] 

    # Read DTC
    # readDTC: 0x19
    # reportDTCByStatusMask: 0x02
    # DTCStatusMask(statusOfDTC) = 0x09 for all DTC, 0x89 for current DTC
    k_req_readDtc = [0x03, 0x19, 0x02, 0x09, 0xAA, 0xAA, 0xAA, 0xAA]

    # Tester Present
    # readDTC: 0x3E
    # response_required: 0x00 = required, 0x80 = not required
    k_req_testerPresent = [0x02, 0x3E, 0x00, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA]
        

# 고정된 메시지들을 미리 만들어 둔다.
# can과 can-fd의 차이를 고려한다.        
k_msg_testerPresent = RawCAN(k_m_id_func, 8, CH1, 0, k_req_testerPresent)
k_msg_extDiag = RawCAN(k_m_id_func, 8, CH1, 0, k_req_extDiag)
k_msg_defDiag = RawCAN(k_m_id_func, 8, CH1, 0, k_req_defDiag)

if k_is_can_fd:
    k_msg_testerPresent = RawCAN(k_m_id_func, 8, CH1, 0, k_req_testerPresent, 0x306)
    k_msg_extDiag = RawCAN(k_m_id_func, 8, CH1, 0, k_req_extDiag, 0x306)
    k_msg_defDiag = RawCAN(k_m_id_func, 8, CH1, 0, k_req_defDiag, 0x306)


# global variables

# 제어기의 응답을 담을 변수 diag_resp를 만든다.
# resp는 제어기 응답 메시지의  m_id를 key로하는 딕셔너리다.
# m_id 별로 아래 항목들이 필요하다.
#   length_response     : 제어기가 알려준 응답의 길리 [바이트] 
#   counter_response    : (응답이 길어 여러 메시지로 나뉘어 전송될 경우) 메시지 카운터
#   serive_id           : 서비스 아이디
#   response_assembled  : (응답이 길어 여러 메시지로 나뉘어 전송될 경우) 응답 메시지들 전체를 조립한 메시지
#   response_hex        : response_assembled의 바이트 하나 하나를 hex로 표현 
diag_resp =  {}

# 현재 m_id를 저장할 변수 m_id_diag_req를 만든다.
# m_id 스캐닝을 할 때 사용한다. 
# 시작값은 k_m_id_scan_start이다.
m_id_diag_req = k_m_id_scan_start

# timer 상태 저장 변수
is_timer_diag_req_on = False
is_timer_tester_present_on = False
is_timer_scan_data_id_on = False

# 현재 req message
req = []

# read data by local id
# data_id 스캐닝을 할 때 사용한다.
data_id_high = 0x00
data_id_low = 0x00

# 현대기술정보 홈페이지에서 복붙한 DTC와 DTC 설명이 있는xlsx 파일을 읽는다.
# xlsx 파일 경로
xlsx_dtc_table = Path('C:\\data\\tosun\\projects\\uds_vehicle_config\\MiniProgram\\PySrc\\베뉴_VDC_DTC_현대기술정보.xlsx')

# DTC표를 Pandas 데이터프레임으로 읽는다.
df_dtc = pd.read_excel(xlsx_dtc_table, index_col='DTC')


# function
def toggle_timer_diag_req() -> None:
    global is_timer_diag_req_on
    
    if is_timer_diag_req_on:
        timer_diag_req.Stop()
        is_timer_diag_req_on = False
        app.log_text('timer_diag_req stoped', lvlOK)
    else:
        timer_diag_req.Start()
        is_timer_diag_req_on = True
        app.log_text('timer_diag_req started', lvlOK)
        
    return    # toggle_timer_diag_req()
    

def toggle_timer_tester_present() -> None:     
    global is_timer_tester_present_on
    
    if is_timer_tester_present_on:
        timer_tester_present.Stop()
        is_timer_tester_present_on = False
        app.log_text('timer_tester_present stopped', lvlOK)
    else:
        timer_diag_req.Start()
        is_timer_tester_present_on = True
        app.log_text('timer_tester_present started', lvlOK)
        
    return    # toggle_timer_tester_present()


def get_dtc_description(dtc):
    '''
    입력으로 받은 DTC에 대한 설명인 description을 반환한다.
    '''

    global df_dtc

    try:
        description = df_dtc.loc[dtc, 'Description']
    except KeyError:
        description = 'DTC를 찾을 수 없습니다.'

    return description


def convert_dtc(dtc_bytes):
    '''
    written by claude.ai
    제어기가 응답한 결함 코드를 표준 코드로 변환한다.
    converts DTC bytes to a standard DTC string
    '''
    
    # DTC 타입 결정을 위한 딕셔너리
    # dictionary to determine DTC type
    dtc_type = {
        0: 'P', 
        1: 'C', 
        2: 'B', 
        3: 'U',
    }
    
    # 첫 번째 바이트의 상위 2비트를 사용하여 DTC 타입 결정
    # determine DTC type using the first byte's upper 2 bits
    first_char = dtc_type[(dtc_bytes[0] >> 6) & 0x3]
    
    # 첫 번째 바이트의 하위 6비트와 두 번째 바이트를 결합하여 중간 부분 생성
    # combine the lower 6 bits of the first byte and the second byte to create the middle part
    middle_part = ((dtc_bytes[0] & 0x3F) << 8) | dtc_bytes[1]
    
    # 세 번째 바이트를 마지막 부분으로 사용
    # use the third byte as the last part
    last_part = dtc_bytes[2]
    
    # DTC 문자열 생성
    # create the DTC string
    dtc_string = f"{first_char}{middle_part:04x}{last_part:02x}"
    
    return dtc_string


def parse_resp(m_id_hex) -> None:
    '''
    제어기의 응답을 분석한다.
    '''

    global diag_resp
    
    # read DTC
    if diag_resp[m_id_hex]['s_id'] == k_resp_s_id_readDtc:    
        # 조립을 완료한 전체 response 메시지를 출력한다.
        app.log_text(f'{diag_resp[m_id_hex]["hex"]}', lvlOK)

        # 처음 3 바이트는 dtc 정보가 아니다.
        # The first 3 bytes are not DTC information.
        # 4 바이트씩 DTC 정보가 있다.
        # DTC information is in 4 bytes each.
        n_dtc = (diag_resp[m_id_hex]['length'] - 3) // 4  
        
        if n_dtc > 0:
            app.log_text(f'{m_id_hex = }', lvlOK)
            app.log_text(f'{n_dtc = }', lvlOK)
            for i_dtc in range(1, n_dtc + 1):
                dtc = diag_resp[m_id_hex]['assembled'][3 + 4 * i_dtc: 3 + 4 * i_dtc + 3]

                dtc_hex = []
                for byte_dtc in dtc:
                    dtc_hex.append(f'{byte_dtc:02x}')
                dtc_hex_joined = ''.join(dtc_hex)

                # DTC가 아니라 fill byte인 경우
                if dtc_hex_joined == 'aaaaaa':
                    continue

                dtc_converted = convert_dtc(dtc)
                dtc_description = get_dtc_description(dtc_converted)
                dtc_state = diag_resp[m_id_hex]['assembled'][3 + 4 * i_dtc + 3] 

                app.log_text(f'{i_dtc = }', lvlOK) 
                app.log_text(f'dtc = {dtc_converted} (0x{dtc_hex_joined})', lvlOK) 
                app.log_text(f'{dtc_description}', lvlOK)             
                app.log_text(f'{dtc_state = :02x}', lvlOK) 
                app.log_text('---', lvlOK) 


    ## read data by identifier
    # DID(Data ID)에 따라 read ECU ID, read Software Version 등 여러 가지가 있다. 
    if diag_resp[m_id_hex]['s_id'] == k_resp_s_id_readDataById:
        # data_id에 따라 처리한다.    
        data_id_high = diag_resp[m_id_hex]['assembled'][1] 
        data_id_low = diag_resp[m_id_hex]['assembled'][2] 

        # data_id 별로 response 처리 방법이 다르다.

        # data_id == ECU Id: 
        if data_id_high == 0xF1 and data_id_low == 0x00: # data_id for ECU Id
            ecu_id = ''.join([chr(c) for c in diag_resp[m_id_hex]['assembled'][3:] if c != 0xaa])
            app.log_text(f'{m_id_hex = }', lvlOK)
            app.log_text(f'data_id = {data_id_high:02X} {data_id_low:02X}', lvlOK)

            # 조립을 완료한 전체 response 메시지를 출력한다.
            app.log_text(f'{diag_resp[m_id_hex]["hex"]}', lvlOK)
            app.log_text(f'{ecu_id = }', lvlOK) 
    
        # data_id == SW Version: 
        elif data_id_high == 0x10 and data_id_low == 0x00:
            sw_ver = ''.join([chr(c) for c in diag_resp[m_id_hex]['assembled'][3:] if c != 0xaa])
            app.log_text(f'{m_id_hex = }', lvlOK)
            app.log_text(f'data_id = {data_id_high:02X} {data_id_low:02X}', lvlOK)

            # 조립을 완료한 전체 response 메시지를 출력한다.
            app.log_text(f'{diag_resp[m_id_hex]["hex"]}', lvlOK)
            app.log_text(f'{sw_ver = }', lvlOK)
        # data_id == whatever    
        else:
            whatever_data = ''.join([chr(c) for c in diag_resp[m_id_hex]['assembled'][3:] if c != 0xaa])
            app.log_text(f'{m_id_hex = }', lvlOK)
            app.log_text(f'data_id = {data_id_high:02X} {data_id_low:02X}', lvlOK)

            # 조립을 완료한 전체 response 메시지를 출력한다.
            app.log_text(f'{diag_resp[m_id_hex]["hex"]}', lvlOK)
            app.log_text(f'{whatever_data = }', lvlOK)


def on_can_diag_resp(ACAN: RawCAN) -> None:
    '''
    Transport Protocol로 분할된 메시지들을 받아서 조립하려면 
    이전 함수 호출 시 데이터들을 기억해야 한다.
    이를 위해 전역 변수를 사용한다. 
    '''
    global diag_resp
    global m_id_diag_req
       
    # response 메시지를 카운트한다.

    # message id
    m_id = ACAN.id

    # diag_resp의 key로 사용하기 위해 int를 str로 변환한다.
    m_id_hex = hex(m_id)
    
    if m_id_hex in diag_resp:
        diag_resp[m_id_hex]['counter'] += 1
    else: 
        #   length_response     : 제어기가 알려준 응답의 길리 [바이트] 
        #   counter_response    : (응답이 길어 여러 메시지로 나뉘어 전송될 경우) 메시지 카운터
        #   serive_id           : 서비스 아이디
        #   response_assembled  : (응답이 길어 여러 메시지로 나뉘어 전송될 경우) 응답 메시지들 전체를 조립한 메시지
        #   response_hex        : response_assembled의 바이트 하나 하나를 hex로 표현 
        diag_resp[m_id_hex] = {}
        diag_resp[m_id_hex]['length'] = 0
        diag_resp[m_id_hex]['counter'] = 1
        diag_resp[m_id_hex]['s_id'] = 0x00
        diag_resp[m_id_hex]['assembled'] = []
        diag_resp[m_id_hex]['hex'] = []


    # 응답의 데이터 8 바이트를 hex로 표시한다.
    # 8은 클래식 CAN의 최대 메시지 길이이고, 
    # 모비스 VDC는 CAN-FD로 통신한다.
    # 진단 통신response 메시지의 길이는 64 바이트이지만,
    # 처음 8 바이트 외에는 0x00으로 채우고 사용하지 않는다.  
    
    data_hex = []
    for i in range(8):
        data_hex.append(f'{ACAN.data[i]:02x}')
    # app.log_text(f'm_id = {m_id_hex:6} counter = {diag_resp[m_id_hex]["counter"]:2}: {data_hex}', lvlOK)
    

    # TP로 전송된 분해된 response를 조립한다.
    pci = ACAN.data[0] & 0xF0       
    if pci == 0x00:        # single frame
        # 같은 m_id의 다음 응답 처리를 위해 초기화 한다.
        # counter는 초기화하지 않는다. 응답 횟수를 추적하기 위해서
        # diag_resp[m_id_hex] = {}
        # diag_resp[m_id_hex]['length'] = 0
        # diag_resp[m_id_hex]['counter'] = 1
        diag_resp[m_id_hex]['s_id'] = 0x00
        diag_resp[m_id_hex]['assembled'] = []
        diag_resp[m_id_hex]['hex'] = []

        diag_resp[m_id_hex]['length'] = ACAN.data[0] & 0x0F
        diag_resp[m_id_hex]['assembled'].extend(ACAN.data[1:8])   
    elif pci == 0x10:      # first frame 
        # 같은 m_id의 다음 응답 처리를 위해 초기화 한다.
        # counter는 초기화하지 않는다. 응답 횟수를 추적하기 위해서
        # diag_resp[m_id_hex] = {}
        # diag_resp[m_id_hex]['length'] = 0
        # diag_resp[m_id_hex]['counter'] = 1
        diag_resp[m_id_hex]['s_id'] = 0x00
        diag_resp[m_id_hex]['assembled'] = []
        diag_resp[m_id_hex]['hex'] = []

        diag_resp[m_id_hex]['length'] = (ACAN.data[0] & 0x0F) * 256 + ACAN.data[1]
        diag_resp[m_id_hex]['assembled'].extend(ACAN.data[2:8])
        
        # NOTE
        # TSMaster의 UDS  기능을 사용하지 않는 경우 
        # flow control 메시지를 전송해야 한다.
        
        # is can_fd?
        if k_is_can_fd:
            # CAN-FD 메시지
            msg_diag_req = RawCAN(m_id_diag_req, 8, CH1, 0, k_flow_control, 0x306)
        else:
            # CAN 메시지
            msg_diag_req = RawCAN(m_id_diag_req, 8, CH1, 0, k_flow_control)
            
        if com.transmit_can_async(msg_diag_req) == 0:
            app.log_text(f'Tx fc = {msg_diag_req}', lvlOK)


    elif (pci == 0x20):    # consecutive frame
        diag_resp[m_id_hex]['assembled'].extend(ACAN.data[1:8])

        # NOTE
        # TSMaster의 UDS  기능을 사용하지 않는다면, 
        # 여기에서 block size를 고려하여 flow control 메시지를 전송해야 한다.
        # block size를 unlimited로 하기 때문에 아무 것도 안 한다.
        
    else:    # flow control 
        pass 
        # do nothing
        # blf replay로 데이터를 처리하는 경우 flow control 메시지를 
        # Rx 할 수도 있다. 
        # 제어기와 온-라인으로 연결한 상태에서는 여기로 올 수 없다.
    

    # TP로 전송된 메시지의 조립이 완료되면 출력한다. 
    if ((diag_resp[m_id_hex]['length'] != 0) and
        (diag_resp[m_id_hex]['length'] <= len(diag_resp[m_id_hex]['assembled']))):

        for byte_response in diag_resp[m_id_hex]['assembled']:
            # byte_response를 hex로 변환하여 diag_resp[m_id_hex]['hex']에 저장한다.
            # 전체 응답 메시지를 hex로 보기 위한 용도이다. 
            diag_resp[m_id_hex]['hex'].append(f'{byte_response:02x}')

        diag_resp[m_id_hex]['s_id'] = diag_resp[m_id_hex]['assembled'][0]
                
        is_negative_resp = (diag_resp[m_id_hex]['s_id'] == 0x7F)
        if not is_negative_resp:
            parse_resp(m_id_hex)
        
    return    # on_can_diag_resp()

# CODE BLOCK END Global_Definitions 

# CODE BLOCK BEGIN Instance 
Instance = MpInstance('mobis_esc_ecu_id_rev_eng')
# CODE BLOCK END Instance 



# CODE BLOCK BEGIN On_Shortcut ctrl_0_init
def on_shortcut_ctrl_0_init() -> None:
    global diag_resp
    global req
    global k_msg_defDiag
    global is_timer_diag_req_on
    global is_timer_tester_present_on
    global is_timer_scan_data_id_on

    # 변수를 초기화한다.
    diag_resp =  {}
    req = []
    app.log_text('initialize variables', lvlOK)

    timer_diag_req.Stop()
    is_timer_diag_req_on = False
    app.log_text('timer_diag_req stopped', lvlOK)
    
    timer_tester_present.Stop()
    is_timer_tester_present_on = False
    app.log_text('timer_tester_present stopped', lvlOK)

    timer_scan_data_id.Stop()
    is_timer_scan_data_id_on = False
    app.log_text('timer_scan_data_id stopped', lvlOK)

    if com.transmit_can_async(k_msg_defDiag) == 0:
        app.log_text('default diag session', lvlOK)

# CODE BLOCK END On_Shortcut ctrl_0_init
# CODE BLOCK BEGIN On_Shortcut ctrl_1_ecu_id
def on_shortcut_ctrl_1_ecu_id() -> None:
    global diag_resp
    global req
    global k_req_readEcuId
    global m_id_diag_req

    # 변수를 초기화한다.
    diag_resp =  {}
    req = k_req_readEcuId # "0x03 0x22 0xF1 0x11 0xAA  0xAA  0xAA"
    
    m_id_diag_req = k_m_id_scan_start
    toggle_timer_diag_req()
    app.log_text('scanning read ECU ID', lvlOK)

# CODE BLOCK END On_Shortcut ctrl_1_ecu_id
# CODE BLOCK BEGIN On_Shortcut ctrl_2_dtc
def on_shortcut_ctrl_2_dtc() -> None:
    global diag_resp
    global req
    global k_req_readDtc
    global m_id_diag_req

    # 변수를 초기화한다.
    diag_resp =  {}
    req = k_req_readDtc
    
    m_id_diag_req = k_m_id_scan_start
    toggle_timer_diag_req()
    app.log_text('scanning read DTC', lvlOK)

# CODE BLOCK END On_Shortcut ctrl_2_dtc
# CODE BLOCK BEGIN On_Shortcut ctrl_3_ext_diag
def on_shortcut_ctrl_3_ext_diag() -> None:
    global diag_resp
    global k_msg_extDiag
    global is_timer_tester_present_on

    # 변수를 초기화한다.
    diag_resp =  {}

    if com.transmit_can_async(k_msg_extDiag) == 0:
        app.log_text('extended diag session', lvlOK)

        timer_tester_present.Start()
        is_timer_tester_present_on = True
        app.log_text('timer_tester_present started', lvlOK)

# CODE BLOCK END On_Shortcut ctrl_3_ext_diag
# CODE BLOCK BEGIN On_Shortcut ctrl_4_sw_ver
def on_shortcut_ctrl_4_sw_ver() -> None:
    global diag_resp
    global req
    global k_req_readSwVer
    global m_id_diag_req

    # 변수를 초기화한다.
    diag_resp =  {}
    req = k_req_readSwVer
    
    m_id_diag_req = k_m_id_scan_start   
    toggle_timer_diag_req()
    app.log_text('scanning read Software Version', lvlOK)

# CODE BLOCK END On_Shortcut ctrl_4_sw_ver
# CODE BLOCK BEGIN On_Shortcut ctrl_5_scan_data_id
def on_shortcut_ctrl_5_scan_data_id() -> None:
    global k_msg_extDiag
    global data_id_low
    global data_id_high
    global m_id_diag_req
    global k_m_id_esc_diag_req
    global is_timer_diag_req_on
    global is_timer_tester_present_on
    global is_timer_scan_data_id_on

    # ESC를 대상으로 한다. 
    # 다른 제어기를 대상으로 할 경우, 
    # 그 제어기의 진단 "요청" m_id를 사용한다. 
    m_id_diag_req = k_m_id_esc_diag_req

    # DID를 스캐닝을 시작할 값으로 초기화한다.
    data_id_high = 0x00
    data_id_low = 0x00

    # data_id 스캔 중에는 timer_diag_req를 정지한다.
    timer_diag_req.Stop()
    is_timer_diag_req_on = False
    app.log_text('timer_diag_req stopped', lvlOK)

    # data_id scan을 위한 readDataById 요청의 주기가 
    # tester_present의 주기보다 짧으면 tester_present 가 필요없다.
    # timer_tester_present를 정지한다.
    if timer_scan_data_id.IntervalUs <= timer_tester_present.IntervalUs:
        timer_tester_present.Stop()
        is_timer_tester_present_on = False
        app.log_text('timer_tester_present stopped', lvlOK)
    
    if com.transmit_can_async(k_msg_extDiag) == 0:
        app.log_text('extended diag session', lvlOK)

        timer_scan_data_id.Start()
        is_timer_scan_data_id_on = True
        app.log_text('timer_scan_data_id started', lvlOK)

# CODE BLOCK END On_Shortcut ctrl_5_scan_data_id
# CODE BLOCK BEGIN Timer timer_diag_req NTAw
timer_diag_req = MpTimer("mobis_esc_ecu_id_rev_eng", "timer_diag_req", 500) # Auto generated code, for modification please goto python code editor
# CODE BLOCK END Timer timer_diag_req
# CODE BLOCK BEGIN Timer timer_tester_present NDAwMA__
timer_tester_present = MpTimer("mobis_esc_ecu_id_rev_eng", "timer_tester_present", 4000) # Auto generated code, for modification please goto python code editor
# CODE BLOCK END Timer timer_tester_present
# CODE BLOCK BEGIN Timer timer_scan_data_id MTAw
timer_scan_data_id = MpTimer("mobis_esc_ecu_id_rev_eng", "timer_scan_data_id", 100) # Auto generated code, for modification please goto python code editor
# CODE BLOCK END Timer timer_scan_data_id
# CODE BLOCK BEGIN On_Timer diag_req dGltZXJfZGlhZ19yZXE_
def on_timer_diag_req() -> None:
    global k_is_can_fd
    global m_id_diag_req
    global req
    global is_timer_diag_req_on
    global is_timer_tester_present_on

    m_id_diag_req += 1

    # diag_req 메시지를 만든다.
    if k_is_can_fd:
        # CAN-FD 메시지
        msg_diag_req = RawCAN(m_id_diag_req, 8, CH1, 0, req, 0x306)
    else:
        # CAN 메시지
        msg_diag_req = RawCAN(m_id_diag_req, 8, CH1, 0, req)

    # diag_req 메시지를 전송한다.
    if com.transmit_can_async(msg_diag_req) == 0:
        # app.log_text(f'{msg_diag_req}', lvlOK)
        # 출력량이 너무 많아서 디버깅할 때만 사용한다.
        # 트레이스 윈도에서 전송 메시지를 확인할 수 있다.
        pass
    
    # 모든 m_id를 처리한 후에 timer_diag_req를 멈춘다.
    if m_id_diag_req >= k_m_id_scan_stop:
        timer_diag_req.Stop()
        is_timer_diag_req_on = False
        app.log_text('timer_diag_req stopped', lvlOK)
        
        timer_tester_present.Stop()
        is_timer_tester_present_on = False
        app.log_text('timer_tester_present stopped', lvlOK)
        
        m_id_diag_req = k_m_id_scan_start

# CODE BLOCK END On_Timer diag_req
# CODE BLOCK BEGIN On_Timer tester_present dGltZXJfdGVzdGVyX3ByZXNlbnQ_
def on_timer_tester_present() -> None:
    global k_msg_testerPresent
    
    if com.transmit_can_async(k_msg_testerPresent) == 0:
        app.log_text(f'testerPresent', lvlOK)

# CODE BLOCK END On_Timer tester_present
# CODE BLOCK BEGIN On_Timer scan_data_id dGltZXJfc2Nhbl9kYXRhX2lk
def on_timer_scan_data_id() -> None:
    global k_is_can_fd
    global k_m_id_esc_diag_req
    global k_req_readDataById
    global data_id_low
    global data_id_high
    global is_timer_scan_data_id_on

    # k_req_readDataById = [0x03, 0x22, 0x01, 0x02, 0xAA, 0xAA, 0xAA, 0xAA] 
    # 0x03: 서비스 요청의 길이, 3 바이트
    # 0x22: readDataByIdentifier 서비스의 Serive Id
    # 0x01: DID의 High Byte 위치
    # 0x02: DID의 Low Byte 위치

    # DID의 High Byte와 Low Byte를 변경한다.
    k_req_readDataById[3] = data_id_low
    k_req_readDataById[2] = data_id_high

    # diag_req 메시지를 만든다.
    # CAN-FD 메시지와 CAN 메시지는 설정이 다르다. 
    if k_is_can_fd:
        # CAN-FD 메시지
        msg_diag_req = RawCAN(k_m_id_esc_diag_req, 8, CH1, 0, k_req_readDataById, 0x306)
    else:
        # CAN 메시지
        msg_diag_req = RawCAN(k_m_id_esc_diag_req, 8, CH1, 0, k_req_readDataById)

    # diag_req 메시지를 전송한다.
    if com.transmit_can_async(msg_diag_req) == 0:
        # app.log_text(f'{msg_diag_req}', lvlOK)
        # 출력량이 너무 많아서 디버깅할 때만 사용한다.
        # 트레이스 윈도에서 전송 메시지를 확인할 수 있다.
        pass
    
    # 모든 DID를 처리한 후에 timer_scan_data_id를 멈춘다.
    if (data_id_low == 0xFF) and (data_id_high == 0xFF):
        timer_scan_data_id.Stop()
        is_timer_scan_data_id_on = False
        app.log_text('timer_scan_data_id stopped', lvlOK)
        
    # DID를 +1만큼 증가한다.
    data_id_low += 1
    if data_id_low > 0xFF:
        data_id_low = 0x00 
        data_id_high += 1

# CODE BLOCK END On_Timer scan_data_id

# CODE BLOCK BEGIN On_CAN_FD_Rx any_message LTEsLTEsLTE_
def on_canfd_rx_any_message(ACAN: RawCAN) -> None:
    # 진단 통신 m_id 범위
    # 0x700 부터 0x7FF 까지
    # k_m_id_diag_lowest = 0x700
    # k_m_id_diag_highest = 0x7FF

    if ((k_m_id_diag_lowest <= ACAN.id) and (ACAN.id <= k_m_id_diag_highest)):
        on_can_diag_resp(ACAN)

# CODE BLOCK END On_CAN_FD_Rx any_message
# CODE BLOCK BEGIN Step_Function  NQ__
def step() -> None:
    pass

# CODE BLOCK END Step_Function 
